package org.example.test.serivce;

import org.example.test.dao.UserDao;
import lombok.Data;
import org.example.test.pojo.UserVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.example.test.pojo.UserDO;
import org.example.test.utils.IdGenerator;

import java.util.Objects;

@Data
public class UserService {

    /**
     * 定义依赖对象
     * 用户DAO
     */
    @Autowired
    private UserDao userDao;

    /**
     * 标识生成器
     */
    @Autowired
    private IdGenerator idGenerator;


    /**
     * 定义依赖参数
     * 可以修改
     */
    @Value("${userService.canModify}")
    private Boolean canModify;


    /**
     * 保存用户
     *
     * @return 用户标识
     * @Param userSave 用户保存
     */
    public Long saveUser(UserVo userSave) {
        // 获取用户标识
        Long userId = userDao.getIdByName(userSave.getName());

        // 根据存在处理
        // 根据存在处理：不存在则创建
        if (Objects.isNull(userId)) {
            // 生成用户标识
            userId = idGenerator.generateId();
            UserDO userDO = new UserDO();
            userDO.setId(userId);
            userDO.setName(userSave.getName());
            userDO.setDescription(userSave.getDescription());
            userDao.create(userDO);
        }
        // 根据存在处理：已存在可修改
        else if (canModify) {
            UserDO userDO = new UserDO();
            userDO.setId(userId);
            userDO.setName(userSave.getName());
            userDO.setDescription(userSave.getDescription());
            userDao.modify(userDO);
        }
        // 根据存在处理：已存在不可修改
        else {
            throw new RuntimeException("不支持修改");
        }
        return userId;
    }

}
