import org.example.test.TestDemoApplication;
import org.example.test.entity.SysRegion;
import org.example.test.serivce.TestService;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.Timeout;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.CsvFileSource;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.jdbc.EmbeddedDatabaseConnection;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.test.context.jdbc.Sql;
import org.springframework.test.web.servlet.MockMvc;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;


@DisplayName("test demo")
@Sql("classpath:sysRegion-h2.sql")
@AutoConfigureTestDatabase(connection = EmbeddedDatabaseConnection.H2)
@AutoConfigureMockMvc
@SpringBootTest(classes = TestDemoApplication.class)
public class SpringBootDemoTest {


    @MockBean
    TestService testService;


    @Autowired
    MockMvc mockMvc;


    @Test
    @DisplayName("Test sayHello")
    public void testSayHello() throws Exception {
        Mockito.doReturn("hello Mockito")
            .when(testService)
            .sayHello();

        mockMvc.perform(get("/test/sayHello"))
            .andExpect(status().isOk())
            .andExpect(content().string("hello Mockito"));
    }


    @Test
    @DisplayName("Test sayHello2")
    public void testSayHello2() throws Exception {
        mockMvc.perform(get("/test/sayHello2"))
            .andExpect(status().isOk())
            .andExpect(jsonPath("$.message").value("Hello"));
    }


    @Test
    @Timeout(5)
    @DisplayName("Test timeout")
    public void testTimeout() throws Exception {
        mockMvc.perform(get("/test/sleep3000"))
            .andExpect(status().isOk());
    }

    @ParameterizedTest
    @DisplayName("Test getSysRegion")
    @ValueSource(strings = {"123456789", "1234567890"})
    public void testGetSysRegion(String code) throws Exception {
        mockMvc.perform(get("/test/getSysRegion").param("code", code))
            .andExpect(status().isOk())
            .andExpect(jsonPath("$.name").value("Mock"));


        mockMvc.perform(get("/test/getSysRegion").param("code", code))
            .andExpect(status().isOk())
            .andExpect(result -> {
                if (!result.getResponse().getContentAsString().contains("Mock")) {
                    throw new RuntimeException("result not contains Mock");
                }
            });


        SysRegion sysRegion = new SysRegion();
        sysRegion.setCode(code);
        mockMvc.perform(
                post("/test/getSysRegion")
                    .content(sysRegion.toJsonString())
                    .contentType(MediaType.APPLICATION_JSON)
            )
            .andExpect(status().isOk())
            .andExpect(jsonPath("$.name").value("Mock"));
    }


    @ParameterizedTest
    @DisplayName("Test getSysRegion2")
    @CsvFileSource(resources = "sysRegionCode.csv")
    public void testGetSysRegion2(String code) throws Exception {
        mockMvc.perform(get("/test/getSysRegion").param("code", code))
            .andExpect(status().isOk())
            .andExpect(jsonPath("$.name").value("Mock"));
    }


}
