import cn.hutool.core.io.IoUtil;
import cn.hutool.core.io.resource.ResourceUtil;
import cn.hutool.json.JSONUtil;
import org.example.test.dao.UserDao;
import org.example.test.pojo.UserDO;
import org.example.test.pojo.UserVo;
import org.example.test.serivce.UserService;
import org.example.test.utils.IdGenerator;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentCaptor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;
import org.powermock.reflect.Whitebox;

import java.io.InputStream;
import java.nio.charset.StandardCharsets;


@ExtendWith(MockitoExtension.class)
public class UserServiceTest {


    /** 定义资源常量 **/
    /**
     * 资源路径
     **/
    private static final String RESOURCE_PATH = "testUserService/";


    @Mock
    UserDao userDao;

    @Mock
    IdGenerator idGenerator;

    @InjectMocks
    UserService userService;


    @BeforeEach
    public void before() {
        Whitebox.setInternalState(userService, "canModify", Boolean.TRUE);
    }

    @Test
    public void testSaveUserWithCreate() {

        // 模拟依赖方法
        // 模拟依赖方法：userDao.getIdByName
        Mockito.doReturn(null)
            .when(userDao)
            .getIdByName(Mockito.anyString());

        Long userId = 123L;
        // 模拟依赖方法：idGenerator.generateId
        Mockito.doReturn(userId)
            .when(idGenerator)
            .generateId();

        // 调用测试方法
        String path = RESOURCE_PATH + "testSaveUserWithCreate/";
        InputStream stream = ResourceUtil.getStream(path + "userSave.json");
        String json = IoUtil.read(stream, StandardCharsets.UTF_8);
        IoUtil.close(stream);
        UserVo userSave = JSONUtil.toBean(json, UserVo.class);
        Assertions.assertEquals(userId, userService.saveUser(userSave), "用户标识不一致");


        //verify(T mock) == verify(mock, times(1))

        // 验证依赖方法
        // 验证依赖方法：userDao.getIdByName
        Mockito.verify(userDao).getIdByName(userSave.getName());
        // 验证依赖方法：idGenerator.generateId
        Mockito.verify(idGenerator).generateId();
        // 验证依赖方法：userDao.create
        ArgumentCaptor<UserDO> userCreateCaptor = ArgumentCaptor.forClass(UserDO.class);
        Mockito.verify(userDao).create(userCreateCaptor.capture());
        stream = ResourceUtil.getStream(path + "userCreate.json");
        json = IoUtil.read(stream, StandardCharsets.UTF_8);
        IoUtil.close(stream);
        Assertions.assertEquals(json, JSONUtil.toJsonStr(userCreateCaptor.getValue()), "用户创建参数不一致");

        // 验证依赖对象
        Mockito.verifyNoMoreInteractions(idGenerator, userDao);
    }
}

